package transport

import (
	"fmt"
	"net/http"

	"github.com/labstack/echo"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/jadwal_sidang/config"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/jadwal_sidang/entity"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/jadwal_sidang/internal/repository"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/jadwal_sidang/internal/usecase"
	"gitlab.informatika.org/k-01-11/graduit-be/src/utils"
)

type AdminJadwalSidangHandler struct {
	jadwalSidangUsecase usecase.JadwalSidangUsecase
}

func NewAdminJadwalSidangHandler(cfg config.JadwalSidangTransportConfig) *AdminJadwalSidangHandler {
	jadwalSidangRepository := repository.NewJadwalSidangRepository(cfg.DBWrite, cfg.DBRead)
	jadwalSidangUsecase := usecase.NewJadwalSidangUseCase(jadwalSidangRepository)

	return &AdminJadwalSidangHandler{
		jadwalSidangUsecase: jadwalSidangUsecase,
	}

}

func (t *AdminJadwalSidangHandler) MountAdmin(group *echo.Group) {
	group.POST("/jadwal-sidang", t.AddJadwalSidang)
	group.GET("/jadwal-sidang-get", t.GetJadwalSidangByBatch)
	group.GET("/jadwal-sidang-latest", t.GetLatestJadwalSidang)
	group.GET("/jadwal-sidang-all", t.GetJadwalSidangAll)
	group.GET("/jadwal-sidang-delete", t.DeleteJadwalSidang)
	group.POST("/jadwal-sidang-edit", t.EditJadwalSidang)
}

func (t *AdminJadwalSidangHandler) AddJadwalSidang(c echo.Context) error {
	request := new(entity.JadwalSidangRequest)
	if err := c.Bind(&request); err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	targetJadwalSidang := &entity.JadwalSidang{
		Name:        request.Name,
		Nim:         request.Nim,
		NamaDosbing: request.NamaDosbing,
		Time:        request.Time,
		NamaDosuji1: request.NamaDosuji1,
		NamaDosuji2: request.NamaDosuji2,
		Ruang:       request.Ruang,
		Judul:       request.Judul,
		IDBatch:     request.IDBatch,
	}

	fmt.Println(targetJadwalSidang)

	newJadwal, err := t.jadwalSidangUsecase.AddJadwalSidang(*targetJadwalSidang)
	if err != nil {
		return c.JSON(http.StatusInternalServerError, utils.ResponseDetailOutput(false, http.StatusInternalServerError, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusCreated, "Jadwal added successfully", newJadwal))
}

func (t *AdminJadwalSidangHandler) GetJadwalSidangByBatch(c echo.Context) error {
	idBatch := c.QueryParam("id_batch")
	listGroupLimit, err := t.jadwalSidangUsecase.GetJadwalSidangByBatch(idBatch)
	if err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "Jadwal Sidang successfully retreived", listGroupLimit))
}

func (t *AdminJadwalSidangHandler) GetLatestJadwalSidang(c echo.Context) error {
	listGroupLimit, err := t.jadwalSidangUsecase.GetLatestJadwalSidang()
	if err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "Latest Jadwal Sidang successfully retreived", listGroupLimit))
}

func (t *AdminJadwalSidangHandler) DeleteJadwalSidang(c echo.Context) error {
	idJadwal := c.QueryParam("id")
	listGroupLimit, err := t.jadwalSidangUsecase.DeleteJadwalSidang(idJadwal)
	if err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "Jadwal Sidang successfully deleted", listGroupLimit))
}

func (t *AdminJadwalSidangHandler) EditJadwalSidang(c echo.Context) error {
	request := new(entity.EditJadwalSidangRequest)
	if err := c.Bind(&request); err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	targetJadwalSidang := &entity.JadwalSidang{
		ID:          request.ID,
		Name:        request.Name,
		Nim:         request.Nim,
		NamaDosbing: request.NamaDosbing,
		Time:        request.Time,
		NamaDosuji1: request.NamaDosuji1,
		NamaDosuji2: request.NamaDosuji2,
		Ruang:       request.Ruang,
		Judul:       request.Judul,
		IDBatch:     request.IDBatch,
	}

	updatedJadwalSidang, err := t.jadwalSidangUsecase.EditJadwalSidang(*targetJadwalSidang)
	if err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "Jadwal Sidang successfully edited", updatedJadwalSidang))
}

func (t *AdminJadwalSidangHandler) GetJadwalSidangAll(c echo.Context) error {
	listGroupLimit, err := t.jadwalSidangUsecase.GetJadwalSidangAll()
	if err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "Jadwal Sidang successfully retreived", listGroupLimit))
}
